@@ -2,15 +2,15 @@
 # this repository contains the full copyright notices and license terms.
 
 from trytond.model import Workflow, ModelView, fields
-from trytond.pool import PoolMeta
+from trytond.pool import PoolMeta, Pool
 from trytond.pyson import Eval, Bool, Id
+from trytond.transaction import Transaction
 from trytond.wizard import Wizard, StateTransition
 
 __all__ = ['PackageType', 'Package', 'ShipmentOut', 'CreateShipping']
 
 
-class PackageType:
-    __metaclass__ = PoolMeta
+class PackageType(metaclass=PoolMeta):
     __name__ = 'stock.package.type'
 
     length = fields.Float('Length', digits=(16, Eval('length_digits', 2)),
@@ -78,8 +78,7 @@ class PackageType:
         return 2
 
 
-class Package:
-    __metaclass__ = PoolMeta
+class Package(metaclass=PoolMeta):
     __name__ = 'stock.package'
 
     shipping_reference = fields.Char('Shipping Reference',
@@ -92,12 +91,21 @@ class Package:
     def search_rec_name(cls, name, clause):
         domain = super(Package, cls).search_rec_name(name, clause)
         return ['OR', domain,
-            ('shipping_reference',) + tuple(clause[:1]),
+            ('shipping_reference',) + tuple(clause[1:]),
             ]
 
+    @classmethod
+    def copy(cls, packages, default=None):
+        if default is None:
+            default = {}
+        else:
+            default = default.copy()
+        default.setdefault('shipping_reference', None)
+        default.setdefault('shipping_label', None)
+        return super(Package, cls).copy(packages, default=default)
 
-class ShipmentOut:
-    __metaclass__ = PoolMeta
+
+class ShipmentOut(metaclass=PoolMeta):
     __name__ = 'stock.shipment.out'
 
     shipping_description = fields.Char('Shipping Description',
@@ -118,7 +126,9 @@ class ShipmentOut:
                     'readonly': (Eval('reference', False)
                         | ~Eval('root_packages', False)
                         | ~Eval('carrier', False)
-                        | (Eval('state') != 'packed')),
+                        | ~Eval('state').in_(['packed', 'done'])),
+                    'depends': ['state', 'carrier', 'reference',
+                        'root_packages'],
                     },
                 })
         cls._error_messages.update({
@@ -128,11 +138,28 @@ class ShipmentOut:
                     ' does not have a carrier set'),
                 })
 
+    @classmethod
+    def __register__(cls, module):
+        pool = Pool()
+        ModelData = pool.get('ir.model.data')
+        model_data = ModelData.__table__()
+        cursor = Transaction().connection.cursor()
+        super(ShipmentOut, cls).__register__(module)
+
+        # Migration from 4.6: rename create_shipping xml id
+        if module == 'stock_package_shipping':
+            cursor.execute(*model_data.update(
+                    [model_data.fs_id],
+                    ['shipment_out_create_shipping_button'],
+                    where=(model_data.module == module)
+                    & (model_data.model == 'ir.model.button')
+                    & (model_data.fs_id == 'create_shipping_button')))
+
     @classmethod
     def search_rec_name(cls, name, clause):
         domain = super(ShipmentOut, cls).search_rec_name(name, clause)
         return ['OR', domain,
-            ('reference',) + tuple(clause[:1]),
+            ('reference',) + tuple(clause[1:]),
             ]
 
     @classmethod
@@ -162,7 +189,7 @@ class ShipmentOut:
         'stock_package_shipping.act_create_shipping_wizard')
     def create_shipping(cls, shipments):
         for shipment in shipments:
-            if shipment.state != 'packed':
+            if shipment.state not in {'packed', 'done'}:
                 cls.raise_user_error('shipment_not_packed', {
                         'shipment': shipment.rec_name,
                         })
